unit mainform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, StdCtrls, ToolWin, ImgList, Grids, mbox, Menus;

type
  TfrmMailer = class(TForm)
    Panel1: TPanel;
    TreeView1: TTreeView;
    Panel2: TPanel;
    RichEdit1: TRichEdit;
    ToolBar1: TToolBar;
    ImageList1: TImageList;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    StringGrid1: TStringGrid;
    Panel3: TPanel;
    StatusBar1: TStatusBar;
    ProgressBar1: TProgressBar;
    PopupMenu1: TPopupMenu;
    ViewHeaders1: TMenuItem;
    MainMenu1: TMainMenu;
    Ayuda1: TMenuItem;
    Acercade1: TMenuItem;
    Servidor1: TMenuItem;
    LeerMensajes1: TMenuItem;
    Correo1: TMenuItem;
    NuevoCorreo1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TreeView1DblClick(Sender: TObject);
    procedure StringGrid1KeyPress(Sender: TObject; var Key: Char);
    procedure TreeView1KeyPress(Sender: TObject; var Key: Char);
    procedure StringGrid1DrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure ViewHeaders1Click(Sender: TObject);
    procedure StringGrid1SelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure StringGrid1DblClick(Sender: TObject);
    procedure Acercade1Click(Sender: TObject);
  private
    { Private declarations }
    MailBoxes : TStrings;
    MBSpecs : TStrings;
    function GetMailboxes( Directory : String; PrettyName : String ): TTreeNode;
    procedure ShowMessageNo( MessageNo : Integer );
    procedure FreeMBoxes;
    procedure UpdateProgress( Current, Max : Integer; Msg : String );
  public
    { Public declarations }
    MailBox : TMBox;
  end;
  TimgList = ( imgMail, imgFolder, imgMBoxNewMail, imbMBox );

var
  frmMailer: TfrmMailer;

const
   C_FormCaption = 'Correito del Tutor Delphi ';

implementation


uses dmMiniMailer, config, headersfrm, acercade;

{$R *.DFM}

procedure TfrmMailer.FormCreate(Sender: TObject);
begin

  // Inicializa algunas cosillas y centra la forma.
  MBSpecs := TStringList.Create;
  MailBoxes := TStringList.Create;
  Caption := C_FormCaption;
  StringGrid1.ColWidths[0] := ImageList1.Width+2;
  Left := ( Screen.Width - Width ) div 2;
  Top :=  ( Screen.Height - Height ) div 2;
  ReadConfiguration(ExtractFilePath(Application.ExeName)+'/MiniMailer.ini', MBSpecs);


end;

procedure TfrmMailer.FormDestroy(Sender: TObject);
begin
   FreeMBoxes;
   // Escribe el archivo de configuracin al final.
   WriteConfiguration(ExtractFilePath(Application.ExeName)+'/MiniMailer.ini', MBSpecs);
   MBSpecs.Free;

   MailBoxes.Free;
end;

procedure TfrmMailer.FormShow(Sender: TObject);
var
  tnTreeNode : TTreeNode;
  i : Integer;
begin

  // Leemos el archivo de configuracin. Tomamos en cuenta cualquier
  // cadena que no comience con "#" (que es el smbolo de "comentarios").
  for i := 0 to MBSpecs.Count - 1 do begin
    MailBoxes.Clear;
    if ( MBSpecs.Names[i] <> '' ) and ( Copy(MBSpecs.Names[i], 1, 1) <>'#' ) then
      GetMailboxes(MBSpecs.Values[MBSpecs.Names[i]], MBSpecs.Names[i]);
  end;

end;

function TfrmMailer.GetMailBoxes( Directory : String; PrettyName : String ): TTreeNode;
var
  i : Integer;
  childNode : TTreeNode;
  MailBox : TMBox;
begin

  // Aqu llenamos el rbol con los MBoxes de un directorio.
  Result := TreeView1.Items.Add( Nil, PrettyName);

  GetMBoxNames(Directory, MailBoxes);
  Result.ImageIndex := Ord(imgFolder);
  Result.SelectedIndex := Ord(imgFolder);
  for i := 0 to MailBoxes.Count -1 do begin
     childNode := TreeView1.Items.AddChild(Result, MailBoxes.Strings[i]);
     childNode.ImageIndex := Ord(imbMBox);
     childNode.SelectedIndex := Ord(imbMBox);
     // Ahora crea unos cuantos MBoxes y asigna los nombres de archivo.
     MailBox := TMBox.Create;
     MailBox.MBoxName := Directory+'/'+ChildNode.Text;
     MailBox.UpdateProc := UpdateProgress;
     childNode.Data := MailBox;

  end;


end;


procedure TfrmMailer.FreeMBoxes;
var
  i : Integer;
begin

   for i := 0 to TreeView1.Items.Count - 1 do begin
     if TreeView1.Items.Item[i].Data <> Nil then TMBox(TreeView1.Items.Item[i].Data).Free;
   end;

end;

procedure TfrmMailer.TreeView1DblClick(Sender: TObject);
var
  i, nMessages : Integer;
begin

  MailBox := Nil;
  if TreeView1.Selected.Data <> Nil then MailBox := TMBox(TreeView1.Selected.Data);
  if MailBox = Nil then exit;

  // Obtiene los mensajes dentro del MBox seleccionado.
  for i := 0 to StringGrid1.ColCount - 1 do StringGrid1.Cols[i].Clear;
  StringGrid1.RowCount := 2;
  StringGrid1.ColWidths[0] := ImageList1.Width+2;
  StringGrid1.Cols[1].Add('From');
  StringGrid1.Cols[2].Add('Subject');
  StringGrid1.Cols[3].Add('Date');
  StringGrid1.Cols[4].Add('MsgId');

  try
    Screen.Cursor := crHourGlass;
    ProgressBar1.Visible := True;
    if MailBox.MessageList.Count = 0 then MailBox.FillMessages;
    nMessages := MailBox.MessageList.Count;
    if nMessages+1 > 1 then StringGrid1.RowCount := nMessages+1
                       else StringGrid1.RowCount := 2;

    for i := 0 to nMessages - 1 do begin
      StringGrid1.Cols[1].Add(MailBox.MessageList.Items[i].MailFrom);
      StringGrid1.Cols[2].Add(MailBox.MessageList.Items[i].MailSubject);
      StringGrid1.Cols[3].Add(MailBox.MessageList.Items[i].MailDate);
      StringGrid1.Cols[4].Add(MailBox.MessageList.Items[i].MailMsgId);
    end;
    StatusBar1.Panels[0].Text := FormatFloat('###,###,###', nMessages)+' messages in '+TreeView1.Selected.Text;

    Caption := C_FormCaption+ ' - '+TreeView1.Selected.Text;

  finally
    Screen.Cursor := crDefault;
    ProgressBar1.Visible := False;
  end;


end;

procedure TfrmMailer.ShowMessageNo( MessageNo : Integer );
begin

  // Muestra un mensaje
  if MailBox = Nil then exit;
  try
    ProgressBar1.Visible := True;
    UpdateProgress( 0, 10, 'Leyendo Correo......');
    Screen.Cursor := crHourglass;
    RichEdit1.Lines.Assign(Mailbox.MessageList.Items[MessageNo].MailBody);
    UpdateProgress( 10, 10, 'Listo.');
  finally
    ProgressBar1.Visible := False;
    Screen.Cursor := crDefault;
  end;

 
end;

procedure TfrmMailer.StringGrid1KeyPress(Sender: TObject; var Key: Char);
begin

  // Abre el mensaje si apretaron "Enter".
  if Key = #13 then StringGrid1DblClick(Sender);

end;

procedure TfrmMailer.TreeView1KeyPress(Sender: TObject; var Key: Char);
begin

  // Lee los mensajes en este mbox si apretaron "Enter".
  if Key = #13 then TreeView1DblClick(Sender);

end;

procedure TfrmMailer.UpdateProgress( Current, Max : Integer; Msg : String );
begin

  ProgressBar1.Max := Max;
  ProgressBar1.Position := Current;
  StatusBar1.Panels.Items[0].Text := Msg;
  StatusBar1.Update;

end;


procedure TfrmMailer.StringGrid1DrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin

  // Muestra un iconito de "correo". Usted puede aadir imgenes al
  // TImageList para que representen diferentes estatus de correo (ledo,
  // no leido, importante, etctera). De esta manera usted puede cambiar
  // el ltimo parmetro en el ImageList para que d informacin til
  // dependiendo en los atributos del TMailMessage actual. 
  if (ACol = 0) and (ARow >0)
  and ( StringGrid1.Cells[ACol+1, ARow] <> '' )
  then begin
    ImageList1.Draw( StringGrid1.Canvas, Rect.Left, Rect.Top, 0);
  end;

end;

procedure TfrmMailer.ViewHeaders1Click(Sender: TObject);
begin

  // Muestra la forma de "Ver encabezados".
  if frmHeaders = Nil then frmHeaders := TfrmHeaders.Create(Self);
  frmHeaders.MailItem := Mailbox.MessageList.Items[StringGrid1.Row-1];
  frmHeaders.Show;

end;

procedure TfrmMailer.StringGrid1SelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
var
  Mail : TMailMessage;
begin

  // Muestra los mensajes mientras navegamos.
  if MailBox.MessageList.Count = 0 then exit;
  Mail := Mailbox.MessageList.Items[ARow-1];
  if Mail = Nil then exit;
  // Ponemos un lmite de mensajes para que los usuarios no tengan que
  // esperar mucho tiempo para ver el mensaje en su pantalla si slo estn
  // moviendo su cursor en el grid.
  if Mail.MailBody.Count < 1000 then ShowMessageNo(ARow-1) else begin
      RichEdit1.Lines.Clear;
      RichEdit1.Lines.Add('Este mensaje es muy largo. Haga doble-click para abrirlo.');
  end;

  // Si la forma de encabezados existe, muestra los encabezados mientras navegamos.
  if frmHeaders <> Nil then frmHeaders.MailItem := Mail;

end;

procedure TfrmMailer.StringGrid1DblClick(Sender: TObject);
begin

 ShowMessageNo(StringGrid1.Row-1);

end;

procedure TfrmMailer.Acercade1Click(Sender: TObject);
begin

  // Muestra nuestra pantalla donde nos hechamos porras. Viva nosotros!!!!
  with TAboutBox.Create(Self) do begin
     ShowModal;
     Free;
  end;

end;

end.
